/*
 * Code Mercenaries Hard- und Software GmbH
 * Karl-Marx-Strasse 147A
 * 12529 Schönefeld
 * Deutschland
 * 
 * August 2015
 * 
 * Used result values:
 *   -1 hard error
 * 	  1 light weight error (like wrong user input)
 *    0 success
 * 
 * This program is using the BCM2835 library for 
 * Raspberry Pi 2. More information at
 * http://www.airspayce.com/mikem/bcm2835/
 * */
 
#include <stdio.h>
#include <string.h>
#include <bcm2835.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>
#include "led_warrior14.h"

void printHelp();
char printStatus();
char ReadArguments(int, char**);
void printCmd();

/*
 * Read in the command line arguments
 * */
char readArguments(int argc, char** argv) {
	
    int buf;
    char mode;
    char result;
	
	//search for help command
    for(int i = 1; i<argc; i++) {
        
        if(strstr(argv[i], "help")) {
            printHelp();
            return 0;
        }
    }
    //started program without parameter
    if(argc == 1) {
        printHelp();
        return 0;
    }
    
    if((argc >= 2) && strcmp(argv[1], "-l") == 0) {
        result = LW14_listDevices();
        if(result != 0) return -1;
        return 0;
    }
    
    //get address for the I2C device
    if((argc >= 3) && (strcmp(argv[1], "-a") == 0)){
        buf = atoi(argv[2]);
        if(buf >= 0 && buf <= 127) {
            LW14_setSlave(buf);
        }else {
            printf("Invalid address! Address has to be between 0 and 127\r\n");
            return 1;
        }
    }else {
        printf("Destination address not given!\r\n");
        return 1;
    }
	
    //i = 3 to start by "real" parameter (argv[0] is the command for starting this program and -a is obligatory)
	for(int i = 3; i<argc; i++) {
		
		if(argv[i][0] == '-' && strlen(argv[i])>1) {
			
			//get parameter
			switch(argv[i][1]) {
                
                //set (W) or get (R) command
                case 'c':	if(i+1 >= argc) return 1;
							
							//first argument
                            switch(argv[++i][0]) {
                                
                                case 'W':   mode = LW14_WRITE;
                                            break;
                                            
                                case 'R':   mode = LW14_READ;
                                            break;
                                            
                                default:    return 1;
                            }
                            
                            //write needs 4 additional arguments
                            if(mode == LW14_WRITE && i+4 >= argc) return 1;
                            
                            //do work
							if(mode == LW14_WRITE) {
								result = LW14_setCmd(atoi(argv[i+1]),
														atoi(argv[i+2]),
														atoi(argv[i+3]),
														atoi(argv[i+4]));
								
								i = i+4;
								
							}else if(mode == LW14_READ) {
								printCmd();
							}
                            
							break;
                
                //change I2C address
                case 'i':	if(++i >= argc) return 1;
							result = LW14_changeToAddressI2C(atoi(argv[i]));
							break;
                
                //get status
                case 's':	result = printStatus();
							break;
                            
                //list all possible devices with addresses
                case 'l':   result = LW14_listDevices();
                            break;
                            
                //no valid parameter, show help
				default:	printHelp();
                            return 0;
                            
			}
			if(result != 0) return -1;
			
		}else {
            return 1;
        }
        
	}
    
	printf("Command accepted\r\n");
	
	return 0;
}

void printHelp() {
	
	printf("Usage:\r\n");
    printf("-l List all active device addresses. Has to be first argument and all other arguments are ignored\r\n\r\n");
    printf("-a x Has to be the first parameter if not using \"-l\" (x from 0 to 127)\r\n\r\n");
    printf("-d addr cmd y s\r\n");
    printf("   (check documentation of LED Warrior14 (chapter 4.3.1) for more information)\r\n\r\n");
    printf("-s Print status\r\n\r\n");
    printf("-i x Set the I2C device address to address x (x from 0 to 127)\r\n\r\n");
    printf("-g W mode_0 addr_0 mode_1 addr_1 Set config\r\n");
    printf("     (check documentation of LED Warrior14 (chapter 4.4) for more information)\r\n\r\n");
    printf("   R Print config\r\n\r\n");
}

char printStatus() {
	
	char status;
	char result = LW14_getStatus(&status);
	if(result != 0) return -1;
	
	//Bus error status
    if((status & 0x80) == 0x80) {
		printf("Status: Bus fault\r\n");
	}else {
		printf("Status: Bus OK\r\n");
	}
	
	//busy or ready
	if((status & 0x40) == 0x40) {
		printf("Status: busy\r\n");
	}else {
		printf("Status: ready\r\n");
	}
	
	//Override
	if((status & 0x20) == 0x20) {
		printf("Override set\n");
	}
	
	//Frame error
	if((status & 0x10) == 0x10) {
		printf("Frame error detected\n");
	}
	
	//Valid reply
	if((status & 0x08) == 0x08) {
		printf("valid reply\n");
	}
	
	//Reply Timeframe
	if((status & 0x04) == 0x04) {
		printf("Reply set\n");
	}
	
	//2 byte telegram received
	if((status & 0x02) == 0x02) {
		printf("2 byte telegram received\r\n");
	}
	
	//1 byte telegram received
	if((status & 0x01) == 0x01) {
		printf("1 byte telegram received\r\n");
	}
	
	return 0;
}

void printCmd() {
	
    char buffer[2] = {0};
    //char result = LW14_getCmd(buffer);
    LW14_getCmd(buffer);
    //if(result != 0) return -1;
    
    printf("Addr: %02x\r\n", buffer[0]);
    printf("Command: %02x\r\n", buffer[1]);
}

char string2Hex(char* value)
{
	int	ret, ret1, exec = 0;
	char newValue[2];

	for(int h=0; h<2; h++)
		newValue[h] = toupper(value[h]);


	switch(newValue[1]) {
		case '0': ret = 0; break;
		case '1': ret = 1; break;
		case '2': ret = 2; break;
		case '3': ret = 3; break;
		case '4': ret = 4; break;
		case '5': ret = 5; break;
		case '6': ret = 6; break;
		case '7': ret = 7; break;
		case '8': ret = 8; break;
		case '9': ret = 9; break;
		case 'A': ret = 10; break;
		case 'B': ret = 11; break;
		case 'C': ret = 12; break;
		case 'D': ret = 13; break;
		case 'E': ret = 14; break;
		case 'F': ret = 15;	break;
		
		default: return 0;
	}

	switch(newValue[0]) {
		
		case '0': ret1 = 0; break;
		case '1': ret1 = 16; break;
		case '2': ret1 = 32; break;
		case '3': ret1 = 48; break;
		case '4': ret1 = 64; break;
		case '5': ret1 = 80; break;
		case '6': ret1 = 96; break;
		case '7': ret1 = 112; break;
		case '8': ret1 = 128; break;
		case '9': ret1 = 144; break;
		case 'A': ret1 = 160; break;
		case 'B': ret1 = 176; break;
		case 'C': ret1 = 192; break;
		case 'D': ret1 = 208; break;
		case 'E': ret1 = 224; break;
		case 'F': ret1 = 240; break;
		
		default: return 0;
	}

	exec = ret + ret1;

	return exec;
}

int main(int argc, char** argv) {
	
	//initialize library for I2C
	if(!bcm2835_init()) {
		printf("No initialization possible!\r\n");
		return 1;
    }
    //begin connection
    bcm2835_i2c_begin();
    bcm2835_i2c_setSlaveAddress(0);
    bcm2835_i2c_setClockDivider(BCM2835_I2C_CLOCK_DIVIDER_2500);

	signed char result = readArguments(argc, argv);
    
    if(result == 1) {
		printf("Error by reading in the arguments!\r\n");
		return 1;
		
	}else if(result == -1) {
		printf("Internal error\r\n");
		return 1;
	}
    
    //end connection
    bcm2835_i2c_end();
    
    return 0;
}
